@@ -21,8 +21,9 @@
 
 #include "dolphin.h"
 
-#include <qdragobject.h>
-#include <qclipboard.h>
+#include <assert.h>
+
+#include <kbookmarkmanager.h>
 #include <kglobal.h>
 #include <kpropertiesdialog.h>
 #include <klocale.h>
@@ -47,14 +48,16 @@
 #include <kstandarddirs.h>
 #include <kprotocolinfo.h>
 #include <kmessagebox.h>
-#include <assert.h>
-#include <krun.h>
 #include <kservice.h>
+#include <kstandarddirs.h>
+#include <krun.h>
+
+#include <qclipboard.h>
+#include <qdragobject.h>
 
 #include "urlnavigator.h"
 #include "viewpropertiesdialog.h"
 #include "viewproperties.h"
-#include "bookmark.h"
 #include "dolphinsettings.h"
 #include "dolphinsettingsdialog.h"
 #include "dolphinstatusbar.h"
@@ -62,6 +65,7 @@
 #include "progressindicator.h"
 #include "dolphinsettings.h"
 #include "sidebar.h"
+#include "sidebarsettings.h"
 
 Dolphin& Dolphin::mainWin()
 {
@@ -192,7 +196,7 @@ void Dolphin::refreshViews()
     assert(m_activeView != 0);
 
     updateViewActions();
-    setActiveView(m_activeView);
+    emit activeViewChanged();
 }
 
 void Dolphin::slotHistoryChanged()
@@ -225,6 +229,13 @@ void Dolphin::slotShowHiddenFilesChanged()
     showHiddenFilesAction->setChecked(m_activeView->isShowHiddenFilesEnabled());
 }
 
+void Dolphin::slotShowFilterBarChanged()
+{
+    KToggleAction* showFilterBarAction =
+        static_cast<KToggleAction*>(actionCollection()->action("show_filter_bar"));
+    showFilterBarAction->setChecked(m_activeView->isFilterBarVisible());
+}
+
 void Dolphin::slotSortingChanged(DolphinView::Sorting sorting)
 {
     KAction* action = 0;
@@ -269,6 +280,8 @@ void Dolphin::slotSelectionChanged()
     compareFilesAction->setEnabled(selectedURLsCount == 2);
 
     m_activeView->updateStatusBar();
+
+    emit selectionChanged();
 }
 
 void Dolphin::closeEvent(QCloseEvent* event)
@@ -277,10 +290,18 @@ void Dolphin::closeEvent(QCloseEvent* event)
     config->setGroup("General");
     config->writeEntry("First Run", false);
 
-    DolphinSettings::instance().save();
+    DolphinSettings& settings = DolphinSettings::instance();
+
+    SidebarSettings* sidebarSettings = settings.sidebar();
+    const bool isSidebarVisible = (m_sidebar != 0);
+    sidebarSettings->setVisible(isSidebarVisible);
+    if (isSidebarVisible) {
+        sidebarSettings->setWidth(m_sidebar->width());
+    }
+
+    settings.save();
 
     config->sync();
-
     KMainWindow::closeEvent(event);
 }
 
@@ -758,25 +779,22 @@ void Dolphin::toggleSortOrder()
 
 void Dolphin::toggleSplitView()
 {
-    // TODO: the sidebar is not used yet
-    //const int sidebarWidth = m_sidebar->width();
-    QValueList<int> sizes;
     if (m_view[SecondaryIdx] == 0) {
+        const int newWidth = (m_view[PrimaryIdx]->width() - m_splitter->handleWidth()) / 2;
+
         // create a secondary view
         m_view[SecondaryIdx] = new DolphinView(m_splitter,
                                                m_view[PrimaryIdx]->url(),
                                                m_view[PrimaryIdx]->mode(),
                                                m_view[PrimaryIdx]->isShowHiddenFilesEnabled());
 
-        // TODO: the sidebar is not used yet
-        const int viewWidth = (m_splitter->width() /*- sidebarWidth*/) / 2;
+        QValueList<int> list = m_splitter->sizes();
+        assert(!list.isEmpty());
+        list.pop_back();
+        list.append(newWidth);
+        list.append(newWidth);
+        m_splitter->setSizes(list);
 
-        // TODO: the sidebar is not used yet
-        //sizes.append(sidebarWidth);
-
-        sizes.append(viewWidth);
-        sizes.append(viewWidth);
-        m_splitter->setSizes(sizes);
         m_view[SecondaryIdx]->show();
     }
     else {
@@ -798,10 +816,6 @@ void Dolphin::toggleSplitView()
             m_view[SecondaryIdx] = 0;
             setActiveView(m_view[PrimaryIdx]);
         }
-        // TODO: the sidebar is not used yet
-        //sizes.append(sidebarWidth);
-        sizes.append(m_splitter->width() /*- sidebarWidth*/);
-        m_splitter->setSizes(sizes);
     }
 }
 
@@ -825,6 +839,14 @@ void Dolphin::showHiddenFiles()
     m_activeView->setShowHiddenFilesEnabled(show);
 }
 
+void Dolphin::showFilterBar()
+{
+    const KToggleAction* showFilterBarAction =
+        static_cast<KToggleAction*>(actionCollection()->action("show_filter_bar"));
+    const bool show = showFilterBarAction->isChecked();
+    m_activeView->slotShowFilterBar(show);
+}
+
 void Dolphin::zoomIn()
 {
     m_activeView->zoomIn();
@@ -1025,6 +1047,35 @@ void Dolphin::addUndoOperation(KIO::Job* job)
     }
 }
 
+void Dolphin::toggleSidebar()
+{
+    if (m_sidebar == 0) {
+        openSidebar();
+    }
+    else {
+        closeSidebar();
+    }
+
+    KToggleAction* sidebarAction = static_cast<KToggleAction*>(actionCollection()->action("sidebar"));
+    sidebarAction->setChecked(m_sidebar != 0);
+}
+
+void Dolphin::closeSidebar()
+{
+    if (m_sidebar == 0) {
+        // the sidebar has already been closed
+        return;
+    }
+
+    // store width of sidebar and remember that the sidebar has been closed
+    SidebarSettings* settings = DolphinSettings::instance().sidebar();
+    settings->setVisible(false);
+    settings->setWidth(m_sidebar->width());
+
+    m_sidebar->deleteLater();
+    m_sidebar = 0;
+}
+
 Dolphin::Dolphin() :
     KMainWindow(0, "Dolphin"),
     m_splitter(0),
@@ -1055,14 +1106,21 @@ void Dolphin::init()
     setAcceptDrops(true);
 
     m_splitter = new QSplitter(this);
-    // TODO: the sidebar is not used yet
-    //m_sidebar = new Sidebar(m_splitter);
-    //connect(m_sidebar, SIGNAL(urlChanged(const KURL&)),
-    //        this, SLOT(slotURLChangeRequest( const KURL& )));
 
-    const QValueList<Bookmark>& bookmarks = DolphinSettings::instance().bookmarks();
+    DolphinSettings& settings = DolphinSettings::instance();
 
-    const KURL& homeURL = bookmarks[0].url();
+    KBookmarkManager* manager = settings.bookmarkManager();
+    assert(manager != 0);
+    KBookmarkGroup root = manager->root();
+    if (root.first().isNull()) {
+        root.addBookmark(manager, i18n("Home"), settings.homeURL(), "folder_home");
+        root.addBookmark(manager, i18n("Storage Media"), KURL("media:/"), "blockdevice");
+        root.addBookmark(manager, i18n("Network"), KURL("remote:/"), "network_local");
+        root.addBookmark(manager, i18n("Root"), KURL("/"), "folder_red");
+        root.addBookmark(manager, i18n("Trash"), KURL("trash:/"), "trashcan_full");
+    }
+
+    const KURL& homeURL = root.first().url();
     setCaption(homeURL.fileName());
     ViewProperties props(homeURL);
     m_view[PrimaryIdx] = new DolphinView(m_splitter,
@@ -1072,14 +1130,15 @@ void Dolphin::init()
 
     m_activeView = m_view[PrimaryIdx];
 
-    // TODO: the sidebar is not used yet
-    //QValueList<int> sizes;
-    //sizes.append(m_sidebar->width());
-    //sizes.append(m_activeView->maximumWidth());
-    //m_splitter->setSizes(sizes);
-
     setCentralWidget(m_splitter);
 
+    // open sidebar
+    SidebarSettings* sidebarSettings = settings.sidebar();
+    assert(sidebarSettings != 0);
+    if (sidebarSettings->isVisible()) {
+        openSidebar();
+    }
+
     setupActions();
     setupGUI(Keys|Save|Create|ToolBar);
     createGUI(0, false);
@@ -1223,7 +1282,7 @@ void Dolphin::setupActions()
                       this, SLOT(showHiddenFiles()),
                       actionCollection(), "show_hidden_files");
 
-    KToggleAction* splitAction = new KToggleAction(i18n("Split View"), "F9",
+    KToggleAction* splitAction = new KToggleAction(i18n("Split View"), "F10",
                                                    this, SLOT(toggleSplitView()),
                                                    actionCollection(), "split_view");
     splitAction->setIcon("view_left_right");
@@ -1246,6 +1305,10 @@ void Dolphin::setupActions()
                 this, SLOT(browse()),
                 actionCollection(), "browse");
 
+    new KToggleAction(i18n("Sidebar"), "F9",
+                      this, SLOT(toggleSidebar()),
+                      actionCollection(), "sidebar");
+
     new KAction(i18n("Adjust View Properties..."), 0,
                 this, SLOT(adjustViewProperties()),
                 actionCollection(), "view_properties");
@@ -1267,6 +1330,10 @@ void Dolphin::setupActions()
                                          actionCollection(), "find_file");
     findFileAction->setIcon("filefind");
 
+    new KToggleAction(i18n("Show Filter Bar"), "filter", "/",
+                      this, SLOT(showFilterBar()),
+                      actionCollection(), "show_filter_bar");
+
     KAction* compareFilesAction = new KAction(i18n("Compare Files"), 0,
                                           this, SLOT(compareFiles()),
                                           actionCollection(), "compare_files");
@@ -1462,12 +1529,19 @@ void Dolphin::updateViewActions()
     slotSortingChanged(m_activeView->sorting());
     slotSortOrderChanged(m_activeView->sortOrder());
 
+    KToggleAction* showFilterBarAction =
+        static_cast<KToggleAction*>(actionCollection()->action("show_filter_bar"));
+    showFilterBarAction->setChecked(m_activeView->isFilterBarVisible());
+
     KToggleAction* showHiddenFilesAction =
         static_cast<KToggleAction*>(actionCollection()->action("show_hidden_files"));
     showHiddenFilesAction->setChecked(m_activeView->isShowHiddenFilesEnabled());
 
     KToggleAction* splitAction = static_cast<KToggleAction*>(actionCollection()->action("split_view"));
     splitAction->setChecked(m_view[SecondaryIdx] != 0);
+
+    KToggleAction* sidebarAction = static_cast<KToggleAction*>(actionCollection()->action("sidebar"));
+    sidebarAction->setChecked(m_sidebar != 0);
 }
 
 void Dolphin::updateGoActions()
@@ -1531,4 +1605,24 @@ void Dolphin::clearStatusBar()
     m_activeView->statusBar()->clear();
 }
 
+void Dolphin::openSidebar()
+{
+    if (m_sidebar != 0) {
+        // the sidebar is already open
+        return;
+    }
+
+    m_sidebar = new Sidebar(m_splitter);
+    m_sidebar->show();
+
+    connect(m_sidebar, SIGNAL(urlChanged(const KURL&)),
+            this, SLOT(slotURLChangeRequest(const KURL&)));
+    m_splitter->setCollapsible(m_sidebar, false);
+    m_splitter->setResizeMode(m_sidebar, QSplitter::KeepSize);
+    m_splitter->moveToFirst(m_sidebar);
+
+    SidebarSettings* settings = DolphinSettings::instance().sidebar();
+    settings->setVisible(true);
+}
+
 #include "dolphin.moc"
