@@ -19,7 +19,10 @@
  ***************************************************************************/
 
 #include "dolphincontextmenu.h"
+
 #include <kactioncollection.h>
+#include <kbookmarkmanager.h>
+#include <kbookmark.h>
 #include <ktrader.h>
 #include <klocale.h>
 #include <krun.h>
@@ -36,7 +39,6 @@
 
 #include "dolphin.h"
 #include "dolphinview.h"
-#include "bookmark.h"
 #include "editbookmarkdialog.h"
 #include "dolphinsettings.h"
 
@@ -148,9 +150,15 @@ void DolphinContextMenu::openViewportContextMenu()
     }
     else if (id == bookmarkID) {
         const KURL& url = dolphin.activeView()->url();
-        Bookmark bookmark(url.filename(), url, "bookmark", true);
-        if (EditBookmarkDialog::getBookmark(bookmark, i18n("Add folder as bookmark"))) {
-            DolphinSettings::instance().addBookmark(bookmark);
+        KBookmark bookmark = EditBookmarkDialog::getBookmark(i18n("Add folder as bookmark"),
+                                                             url.filename(),
+                                                             url,
+                                                             "bookmark");
+        if (!bookmark.isNull()) {
+            KBookmarkManager* manager = DolphinSettings::instance().bookmarkManager();
+            KBookmarkGroup root = manager->root();
+            root.addBookmark(manager, bookmark);
+            manager->emitChanged(root);
         }
     }
 
@@ -222,9 +230,15 @@ void DolphinContextMenu::openItemContextMenu()
 
     if (id == bookmarkID) {
         const KURL selectedURL(m_fileInfo->url());
-        Bookmark bookmark(selectedURL.filename(), selectedURL, "bookmark", true);
-        if (EditBookmarkDialog::getBookmark(bookmark, i18n("Add folder as bookmark"))) {
-            DolphinSettings::instance().addBookmark(bookmark);
+        KBookmark bookmark = EditBookmarkDialog::getBookmark(i18n("Add folder as bookmark"),
+                                                             selectedURL.filename(),
+                                                             selectedURL,
+                                                             "bookmark");
+        if (!bookmark.isNull()) {
+            KBookmarkManager* manager = DolphinSettings::instance().bookmarkManager();
+            KBookmarkGroup root = manager->root();
+            root.addBookmark(manager, bookmark);
+            manager->emitChanged(root);
         }
     }
     else if (id >= actionsIDStart) {
@@ -326,7 +340,7 @@ void DolphinContextMenu::insertActionItems(KPopupMenu* popup,
     QStringList dirs = KGlobal::dirs()->findDirs("data", "dolphin/servicemenus/");
 
     KPopupMenu* menu = 0;
-    for (QStringList::ConstIterator dirIt = dirs.begin(); dirIt != dirs.end(); ++dirIt ) {
+    for (QStringList::ConstIterator dirIt = dirs.begin(); dirIt != dirs.end(); ++dirIt) {
         QDir dir(*dirIt);
         QStringList entries = dir.entryList("*.desktop", QDir::Files);
 
