@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2006 by Peter Penz                                      *
- *   peter.penz@gmx.at                                                     *
+ *   Copyright (C) 2006 by Peter Penz <peter.penz@gmx.at>                  *
+ *   Copyright (C) 2006 by Gregor Kališnik <gregor@podnapisi.net>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -39,6 +39,8 @@
 #include "renamedialog.h"
 #include "progressindicator.h"
 
+#include "filterbar.h"
+
 DolphinView::DolphinView(QWidget *parent,
                          const KURL& url,
                          Mode mode,
@@ -52,7 +54,8 @@ DolphinView::DolphinView(QWidget *parent,
     m_statusBar(0),
     m_iconSize(0),
     m_folderCount(0),
-    m_fileCount(0)
+    m_fileCount(0),
+    m_filterBar(0)
 {
     setFocusPolicy(QWidget::StrongFocus);
     m_topLayout = new QVBoxLayout(this);
@@ -96,11 +99,20 @@ DolphinView::DolphinView(QWidget *parent,
             this, SLOT(slotErrorMessage(const QString&)));
     connect(m_dirLister, SIGNAL(refreshItems (const KFileItemList&)),
             this, SLOT(slotRefreshItems(const KFileItemList&)));
+    connect(m_dirLister, SIGNAL(newItems(const KFileItemList&)),
+           this, SLOT(slotAddItems(const KFileItemList&)));
 
     m_iconSize = KIcon::SizeMedium;
 
     m_topLayout->addWidget(m_urlNavigator);
     createView();
+
+    m_filterBar = new FilterBar(this);
+    m_filterBar->hide();
+    m_topLayout->addWidget(m_filterBar);
+    connect(m_filterBar, SIGNAL(signalFilterChanged(const QString&)),
+           this, SLOT(slotChangeNameFilter(const QString&)));
+
     m_topLayout->addWidget(m_statusBar);
 }
 
@@ -352,6 +364,10 @@ void DolphinView::updateStatusBar()
     }
 }
 
+void DolphinView::requestItemInfo(const KURL& url)
+{
+    emit signalRequestItemInfo(url);
+}
 
 bool DolphinView::isURLEditable() const
 {
@@ -792,6 +808,12 @@ void DolphinView::slotRefreshItems(const KFileItemList& /* list */)
     QTimer::singleShot(0, this, SLOT(reload()));
 }
 
+void DolphinView::slotAddItems(const KFileItemList& list)
+{
+  fileView()->addItemList(list);
+  fileView()->updateView();
+}
+
 void DolphinView::slotGrabActivation()
 {
     Dolphin::mainWin().setActiveView(this);
@@ -1009,4 +1031,48 @@ QString DolphinView::renameIndexPresentation(int index, int itemCount) const
     return str;
 }
 
+void DolphinView::slotShowFilterBar(bool show)
+{
+    assert(m_filterBar != 0);
+    if (show) {
+        m_filterBar->show();
+    }
+    else {
+        m_filterBar->hide();
+    }
+}
+
+void DolphinView::slotChangeNameFilter(const QString& nameFilter)
+{
+    // The name filter of KDirLister does a 'hard' filtering, which
+    // means that only the items are shown where the names match
+    // exactly the filter. This is non-transparent for the user, which
+    // just wants to have a 'soft' filtering: does the name contain
+    // the filter string?
+    QString adjustedFilter(nameFilter);
+    adjustedFilter.insert(0, '*');
+    adjustedFilter.append('*');
+
+    m_dirLister->setNameFilter(adjustedFilter);
+    m_dirLister->emitChanges();
+
+    // TODO: this is a workaround for QIconView: the item position
+    // stay as they are by filtering, only an inserting of an item
+    // results to an automatic adjusting of the item position. In Qt4/KDE4
+    // this workaround should get obsolete due to Interview.
+    KFileView* view = fileView();
+    if (view == m_iconsView) {
+        KFileItem* first = view->firstFileItem();
+        if (first != 0) {
+            view->removeItem(first);
+            view->insertItem(first);
+        }
+    }
+}
+
+bool DolphinView::isFilterBarVisible() const
+{
+  return m_filterBar->isVisible();
+}
+
 #include "dolphinview.moc"
